package com.example.shooter;

import com.example.shooter.timer.MyTimer;
import com.example.shooter.timer.Updatable;
import javafx.application.Application;
import javafx.scene.Cursor;
import javafx.scene.Group;
import javafx.scene.Scene;
import javafx.scene.input.KeyEvent;
import javafx.scene.input.MouseEvent;
import javafx.scene.paint.Color;
import javafx.scene.transform.Rotate;
import javafx.scene.transform.Translate;
import javafx.stage.Stage;

import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

import static com.example.shooter.Constants.*;

public class JavaFXApplication extends Application {
	
	@Override
	public void start ( Stage stage ) throws IOException {
		Translate fieldPosition = new Translate ( WINDOW_WIDTH / 2, WINDOW_HEIGHT / 2 );
		Field field = new Field ( FIELD_RADIUS, fieldPosition );
		
		field.setFill ( Color.GREEN );
		Translate playerPosition = new Translate ( WINDOW_WIDTH / 2, WINDOW_HEIGHT / 2 );
		Player player = new Player ( PLAYER_RADIUS, playerPosition );
		
		Group root = new Group ( field, player );
		
		List<Enemy>  enemies = new ArrayList<> ( );
		for ( int i = 0; i < NUMBER_OF_ENEMIES; ++i ) {
			Enemy enemy = new Enemy ( ENEMY_RADIUS );
			
			enemy.getTransforms ( ).addAll (
					new Translate ( WINDOW_WIDTH / 2, WINDOW_HEIGHT / 2 ),
					new Rotate ( 360. / NUMBER_OF_ENEMIES * i ),
					new Translate ( ENEMY_PLACEMENT_RADIUS, 0 )
			);
			
			root.getChildren ( ).add ( enemy );
			
			enemies.add ( enemy );
		}
		
		Scene scene = new Scene ( root, WINDOW_WIDTH, WINDOW_HEIGHT );
		
		MyTimer timer = new MyTimer ( );
		timer.start ( );
		
		scene.addEventHandler ( KeyEvent.KEY_PRESSED, keyEvent -> {
			double stepX = 0;
			double stepY = 0;
			switch ( keyEvent.getCode ( ) ) {
				case UP: {
					stepY = -PLAYER_STEP;
					break;
				}
				case DOWN: {
					stepY = PLAYER_STEP;
					break;
				}
				case LEFT: {
					stepX = -PLAYER_STEP;
					break;
				}
				case RIGHT: {
					stepX = PLAYER_STEP;
					break;
				}
			}
			
			if ( stepX != 0 || stepY != 0 ) {
				player.move ( stepX, stepY, field );
			}
		});
		
		scene.addEventHandler ( MouseEvent.ANY, mouseEvent -> {
			Bullet bullet = player.handleMouseEvent ( mouseEvent );
			
			if ( bullet != null ) {
				root.getChildren ( ).add ( bullet );
				
				Updatable updatable = dns -> {
					boolean outOfBounds = bullet.update (
							dns,
							0,
							WINDOW_WIDTH,
							0,
							WINDOW_HEIGHT
					);
					
					boolean collided = false;
					if ( outOfBounds ) {
						root.getChildren ( ).remove ( bullet );
					} else {
						for ( int i = 0; i < enemies.size ( ); ++i ) {
							collided = enemies.get ( i ).handleCollision ( bullet );
							if ( collided ) {
								root.getChildren ( ).remove ( enemies.get ( i ) );
								root.getChildren ( ).remove ( bullet );
								enemies.remove ( i );
								break;
							}
						}
					}
					
					return outOfBounds || collided;
				};
				
				timer.add ( updatable );
			}
		} );
		
		scene.setFill ( Color.LIGHTBLUE );
		scene.setCursor ( Cursor.NONE );
		stage.setTitle ( "Shooter" );
		stage.setScene ( scene );
		stage.setResizable ( false );
		stage.show ( );
	}
	
	public static void main ( String[] args ) {
		launch ( );
	}
}