package com.example.shooter;

import javafx.geometry.Point2D;
import javafx.scene.Group;
import javafx.scene.input.MouseEvent;
import javafx.scene.paint.Color;
import javafx.scene.shape.Circle;
import javafx.scene.shape.Rectangle;
import javafx.scene.transform.Rotate;
import javafx.scene.transform.Translate;

public class Player extends Group {
	private double radius;
	private double mouseX;
	private double mouseY;
	private Translate position;
	private Rotate    rotate;

	public Player ( double radius, Translate position ) {
		this.radius   = radius;
		this.position = position;
		
		Circle body = new Circle ( radius, Color.YELLOW );
		
		this.rotate = new Rotate ( );
		
		final double gunWidth = 0.3 * radius;
		final double gunHeight = 2 * radius;
		Rectangle gun = new Rectangle ( gunWidth, gunHeight );
		gun.setFill ( Color.PURPLE );
		gun.getTransforms ( ).addAll (
				this.rotate,
				new Translate ( -gunWidth / 2, 0 )
		);
		
		super.getChildren ( ).addAll ( body, gun );
		
		super.getTransforms ( ).add ( position );
	}
	
	private void updateGun ( ) {
		Point2D vector = new Point2D (
				this.mouseX - this.position.getX ( ),
				this.mouseY - this.position.getY ( )
		).normalize ( );
		
		double angle = -Math.signum ( vector.getX ( ) ) * vector.angle ( 0, 1 );
		this.rotate.setAngle ( angle );
	}
	
	public void move ( double stepX, double stepY, Field field ) {
		double newX = this.position.getX ( ) + stepX;
		double newY = this.position.getY ( ) + stepY;
		
		double fieldX = field.getPosition ( ).getX ( );
		double fieldY = field.getPosition ( ).getY ( );
		
		double dx = newX - fieldX;
		double dy = newY - fieldY;
		double dr = field.getRadius ( ) - this.radius * 0.5;
		
		double distanceSquared = dx * dx + dy * dy;
		
		if ( distanceSquared <= dr * dr ) {
			this.position.setX ( newX );
			this.position.setY ( newY );
			
			this.updateGun ( );
		}
	}
	
	public Bullet handleMouseEvent ( MouseEvent mouseEvent ) {
		this.mouseX = mouseEvent.getX ( );
		this.mouseY = mouseEvent.getY ( );
		
		this.updateGun ( );
		
		Bullet bullet = null;
		
		if ( mouseEvent.getEventType ( ).equals ( MouseEvent.MOUSE_PRESSED ) && mouseEvent.isPrimaryButtonDown ( ) ) {
			Point2D speed = new Point2D (
					this.mouseX - this.position.getX ( ),
					this.mouseY - this.position.getY ( )
			).normalize ( );
			
			Point2D offset = speed.multiply ( 2.1  * radius );
			
			double x = this.position.getX ( ) + offset.getX ( );
			double y = this.position.getY ( ) + offset.getY ( );
			
			Translate position = new Translate ( x, y );
			
			bullet = new Bullet ( 0.3 * radius, position, speed );
		}
		
		return bullet;
	}
}
